# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil
import time
import time

from ksearch.crawler import database

class DatabaseTestCase(unittest.TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp()
        self.database = database.Database(self.tmp_dir)

    def tearDown(self):
        del self.database
        shutil.rmtree(self.tmp_dir)

    def test_get(self):
        ret = self.database.get("1234", 1234)
        self.assertEquals(ret, None)

    def test_items(self):
        self.database.put('test', 'foo', 1)
        self.database.put('test', 'bar', 2)
        items = self.database.items('test')
        self.assertEquals(items, [('bar', '2'), ('foo', '1')])

    def test_get_all(self):
        namespace = 'test text'
        key = 'test key'
        data1 = 'test data1'
        data2 = 'test data2'
        i = self.database.put(namespace, key, data1)
        time.sleep(1.1)
        j = self.database.put(namespace, key, data2)
        tmp = self.database.get_all(namespace, key)
        ret = [x[1] for x in tmp]
        self.assertEquals(ret, [data1, data2])

    def test_get_index_range(self):
        namespace = 'get_index_range_test'
        key = 'test'
        self.database.put(namespace, key, '0', index=0)
        self.database.put(namespace, key, '1', index=1)
        self.database.put(namespace, key, '2', index=2)
        self.database.put(namespace, key, '3', index=3)

        start = 0
        end = 4
        data = self.database.get_index_range(namespace, key, start, end)

        self.assertEquals(data[0][1], '0')
        self.assertEquals(data[1][1], '1')
        self.assertEquals(data[2][1], '2')
        self.assertEquals(data[3][1], '3')

        start = 1
        end = 2
        data = self.database.get_index_range(namespace, key, start, end)

        self.assertEquals(data[0][1], '1')
        self.assertEquals(data[1][1], '2')

    def test_put(self):
        namespace1 = 'test text1'
        namespace2 = 'test text2'
        key = 'test key'
        data = 'test data'
        i = self.database.put(namespace1, key, data)
        ret = self.database.get(namespace1, key)
        self.assertEquals(ret, data)
        ret = self.database.get(namespace2, key)
        self.assertEquals(ret, None)

    def test_increment_decrement(self):
        namespace = 'increment_test'
        key = 'test'
        self.database.increment(namespace, key)
        ret = self.database.get(namespace, key)
        self.assertEquals(ret, '1')

        self.database.decrement(namespace, key)
        ret = self.database.get(namespace, key)
        self.assertEquals(ret, '0')

        key = 'test2'
        self.database.decrement(namespace, key)
        ret = self.database.get(namespace, key)
        self.assertEquals(ret, '-1')

def suite():
    return unittest.TestLoader().\
        loadTestsFromTestCase(DatabaseTestCase)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())   #pragma: no cover
