#-*- coding: utf-8 -*-
"""Crawler module."""

import urllib2, re
import os, sys, urlparse
import operator
import time, sets
import logging
import traceback

import ksearch.util

check = sets.Set() 
avoid_prefix = ['javascript:', 'mailto:']
avoid_suffix = ['.gif', '.jpg', '.png', '.swf', '.css', '.js']

class Downloader(object):
    """Crawler class."""
    def __init__(self, scheduler, storage, database, idx_queue):
        """
        Initializes the Downloader.

        @type  scheduler: Scheduler
        @param scheduler: Scheduler object.
        @type  storage: Storage
        @param storage: Storage object.
        @type  database: Database 
        @param database: Database object.
        @type  idx_queue: Queue
        @param idx_queue: Indexing queue object.
        """
        self.scheduler = scheduler
        self.storage = storage
        self.database = database
        self.idx_queue = idx_queue
        self.logger = logging.getLogger('crawler')

    def writefile(self, url, data, type):
        """
        Saves the web page and puts it into the indexing queue.

        @type  url: string
        @param url: URL to add.
        @type  data: string
        @param data: Data of the URL.
        @type  type: string
        @param type: MIME type of the data.
        """
        # doc_id = 문서에 대한 유니크한 아이디
        doc_id = self.storage.get_id(url)
        if doc_id == None: 
            doc_id = self.storage.add(url, data, type)
            self.idx_queue.append(str(doc_id))

            # 통계를 위해 데이터베이스에 입력
	    self.database.increment("statistics", "total_stored_page")
            hostname = urlparse.urlparse(url)[1]
	    self.database.increment("stored_pages", hostname)
        else:
            self.storage.modify(doc_id, data)

    def crawl(self, url):
        """
        Crawl the URL.

        @type  url: string
        @param url: URL to crawl.
        """
        try:
            handle = urllib2.urlopen(url)
            data = handle.read(1024*1024)
            type = handle.info().gettype()
        except Exception:
            self.logger.info("crawl error url: %s\nDetailed information:\n%s", 
                    url, traceback.format_exc(2))
            self.scheduler.put_blacklist(url)
            return

        self.writefile(url, data, type)

        start = time.time()
        outlinks = ksearch.util.extract_links(data, base_url=url)
        for i, outlink in enumerate(outlinks):
            self.database.put("outlinks", url, outlink, index=i)
        duration = time.time() - start
        self.database.put("statistics", "crawl_outlinks_time", duration)

        for next_url in outlinks:
            avoid = False
            for prefix in avoid_prefix:
                if next_url.lower().startswith(prefix):
                    avoid = True
                    break
            if avoid:
                continue
            for suffix in avoid_suffix:
                if next_url.lower().endswith(suffix):
                    avoid = True
                    break
            if avoid:
                continue

            if (next_url not in check):
                check.add(next_url)
                self.scheduler.put(next_url)

    def start(self):
        """
        Start crawling.
        """
        while True:
            url = self.scheduler.get()
            if not url:
                self.logger.info("No url received from scheduler. Sleeping...")
                time.sleep(5)
                continue
            # Ad-hoc blacklisting.
            if 'acm.kaist.ac.kr' in url:
                continue
            if not url:
                time.sleep(1)
                continue
            self.logger.info("crawling url: %s", url)
            start = time.time()
            self.crawl(url)
            duration = time.time() - start
            self.database.put("statistics", "crawl_time", duration)

if __name__ == "__main__":
    main()
