#-*- coding: utf-8 -*-

import sys
import bsddb
import os
import threading
import copy

import Pyro.util
import Pyro.core


class BTreeQueue(object):
    """ B+ Tree 기반의 큐 """
    def __init__(self, db_dir, database=None):
        """ 큐 초기화 """
        self.logger = Pyro.util.UserLogger()
        env = bsddb.db.DBEnv()
        env.open(db_dir, bsddb.db.DB_CREATE
                         |bsddb.db.DB_INIT_MPOOL
                         |bsddb.db.DB_INIT_LOG
                         |bsddb.db.DB_INIT_TXN
                         |bsddb.db.DB_THREAD
                         #|bsddb.db.DB_RECOVER
                         )
        self.db = bsddb.db.DB(env)
        self.db.open('urlqueue.db', 'urlqueue', dbtype=bsddb.db.DB_BTREE, 
                     flags=bsddb.db.DB_CREATE
                            |bsddb.db.DB_THREAD
                            )

        self.db_name = db_dir.rsplit('_', 1)[0]
        self.total_key = 'total_%s_size' % self.db_name
        self.database = database
        self.lock = threading.RLock()

        cursor = self.db.cursor()
        r = cursor.last()
        if not r:
            self.last_id = 100000  # XXX(serialx): Crude hack!!
        else:
            self.last_id = int(r[0])
        cursor.close()

    def __del__(self):
        """ 큐 서비스 종료 """
        self.close()

    def close(self):
        self.db.close()

    def prepend(self, txt):
        """ 큐의 맨 앞에 원소 추가 """
        self.lock.acquire()
        cursor = self.db.cursor()
        ret = cursor.first()
        if ret == None:
            self.append(txt)
            cursor.close()
            self.lock.release()
            return
        id = int(ret[0])
        self.db.put("%020d"%(id-1), str(txt))
        cursor.close()
        self.lock.release()

        if self.database:
            if isinstance(self.database, Pyro.core.DynamicProxy):
                database = copy.copy(self.database)  # Pyro proxy thread safety
            else:
                database = self.database
            count = self.db.stat()['ndata']
            database.put("statistics", self.total_key, count)

    def append(self, txt):
        """ 큐에 원소 추가 """
        self.lock.acquire()
        self.db.put("%020d"%(self.last_id+1), str(txt))
        self.last_id += 1
        self.lock.release()

        if self.database:
            if isinstance(self.database, Pyro.core.DynamicProxy):
                database = copy.copy(self.database)  # Pyro proxy thread safety
            else:
                database = self.database
            count = self.db.stat()['ndata']
            database.put("statistics", self.total_key, count)

    def consume(self):
        """ 큐에서 원소 추출 """
        self.lock.acquire()
        try:
            cursor = self.db.cursor()
            ret = cursor.first()
            if ret != None:
                cursor.delete()
                if self.database:
                    if isinstance(self.database, Pyro.core.DynamicProxy):
                        database = copy.copy(self.database)  # Pyro proxy thread safety
                    else:
                        database = self.database
                    count = self.db.stat()['ndata']
                    database.put("statistics", self.total_key, count)
                cursor.close()
                self.lock.release()
                return ret[1]
            cursor.close()
            self.lock.release()
            return None
        except bsddb.db.DBKeyEmptyError, e:
            self.logger.msg("Queue.consume: bsddb.db.DBKeyEmptyError occured!")
            cursor.close()
            self.lock.release()
            return None

    def empty(self):
        """ 큐가 비었는지 검사 """
        self.lock.acquire()
        cursor = self.db.cursor()
        ret = cursor.first()
        cursor.close()
        self.lock.release()
        return (ret == None)
