# -*- coding: utf-8 -*-

import Queue
import sys
import time
import urlparse
import socket
import logging
import bsddb
import copy

import Pyro.core

class Scheduler(object):
    """ 스케쥴러 """
    def __init__(self, queue, storage, database, db_path):
        """ 스케쥴러 초기화 """
        self.logger = logging.getLogger("crawler")
        self.source = queue
        self.storage = storage
        self.database = database
        self.queue = Queue.Queue()

        self.env = bsddb.db.DBEnv()
        self.env.set_flags(bsddb.db.DB_TXN_NOSYNC, 1)
        self.env.open(db_path, bsddb.db.DB_CREATE
                              |bsddb.db.DB_INIT_LOCK
                              |bsddb.db.DB_THREAD
                              |bsddb.db.DB_INIT_MPOOL
                              |bsddb.db.DB_INIT_LOG
                              |bsddb.db.DB_INIT_TXN
                              |bsddb.db.DB_RECOVER
                              )

        self.refresh_db = bsddb.db.DB(self.env)
        self.refresh_db.open('refresh.db', 'refresh', dbtype=bsddb.db.DB_BTREE,
                        flags=bsddb.db.DB_CREATE
                               |bsddb.db.DB_THREAD
                               |bsddb.db.DB_AUTO_COMMIT
                               )

    def __del__(self):
        self.close()

    def close(self):
        self.refresh_db.close()

    def put_blacklist(self, url):
        """ 존재하지 않는 URL 리스트에 추가 - 일주일 뒤에 크롤링 """
        next = int(time.time())+604800
        self.refresh_db.put("%020d"%next, url)

    def is_kaist(self, url):
        """ 해당 URL이 KAIST 내부 웹인지 검사 """
        hostname = urlparse.urlparse(url)[1]
        try:
            ip = socket.gethostbyname(hostname)
        except socket.gaierror:
            return False
        return ip.startswith('143.248.')

    def put(self, url):
        """ 스케쥴링 큐에 추가 """
        if self.is_kaist(url):
            self.source.append(url)

    def put_front(self, url):
        """ 스케쥴링 큐의 맨 앞에 추가 """
        if self.is_kaist(url):
            self.source.prepend(url)

    def apply(self, url, ip):
        """ 3.5일 뒤에 다시 크롤링하도록 refresh 리스트에 추가 """
        next = int(time.time())
        self.add_ip(ip, next)
        next += 302400
        self.refresh_db.put("%020d"%next, url)

    def add_ip(self, ip, time):
        """ 해당 IP를 마지막으로 크롤링한 시간 저장 """
        if ip==None: return
        if isinstance(self.database, Pyro.core.DynamicProxy):
            database = copy.copy(self.database)  # Pyro proxy thread safety
        else:
            database = self.database
        database.put("ip_crawl", ip, time)

    def get_time(self, ip):
        """ 해당 IP를 마지막으로 크롤링한 시간 리턴 """
        if ip==None: return
        if isinstance(self.database, Pyro.core.DynamicProxy):
            database = copy.copy(self.database)  # Pyro proxy thread safety
        else:
            database = self.database
        return database.get("ip_crawl", ip)

    def get(self):
        """ 크롤링해야 할 URL을 선택하여 다운로더에 전달 """
        #refresh 할 때가 된 url 선택
        start = time.time()
        txn = self.env.txn_begin()
        url = None
        try:
            cursor = self.refresh_db.cursor()
            next = cursor.first()
            if next != None and int(next[0]) <= int(time.time()):
                url = next[1]
                hostname = urlparse.urlparse(url)[1]
                ip = socket.gethostbyname(hostname)
                cursor.delete(txn=txn)
                self.apply(url, ip)
            else:
                url = None
        except:
            txn.abort()
        else:
            txn.commit()
        if url:
            return url
        cursor.close()


        #큐에 50개 이하의 url이 남아있다면 소스에서 50개 보충한다.
        if self.queue.qsize() < 5:
            for i in range(5):
                tmp = self.source.consume()
                if tmp == None: break
                self.queue.put(tmp)

        #마지막 request 이후 5초 이상 지난 url만을 선택
        while not self.queue.empty():
            url = self.queue.get()
            hostname = urlparse.urlparse(url)[1]
            try:
                ip = socket.gethostbyname(hostname)
            except socket.gaierror:
                self.logger.error("No address associated with hostname: %s", hostname)
                continue
            past = self.get_time(ip)
            if past == None: break
            if int(time.time()) - int(past) >= 10: break
            self.source.append(url)
        else:
            url = None

        #마지막으로 request한 시간을 갱신하고 리턴
        if url != None:
            self.apply(url, ip)
        if not url:
            url = self.source.consume()
        return url
