# -*- coding: utf-8 -*-

import unittest
import tempfile
import shutil
import socket
import time

from ksearch.crawler import scheduler
from ksearch.crawler import queue
from ksearch.crawler import storage
from ksearch.crawler import database

def fakeip(domain):
    return domain

class SchedulerTestCase(unittest.TestCase):
    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp()
        self.queue = queue.BTreeQueue(self.tmp_dir)
        self.tmp_dir = tempfile.mkdtemp()
        self.storage = storage.Storage(self.tmp_dir)
        self.tmp_dir = tempfile.mkdtemp()
        self.database = database.Database(self.tmp_dir)
        self.tmp_dir = tempfile.mkdtemp()
        self.scheduler = scheduler.Scheduler(self.queue, self.storage, self.database, self.tmp_dir)

    def tearDown(self):
        del self.scheduler
        del self.queue
        shutil.rmtree(self.tmp_dir)

    def test_put_blacklist(self):
        rets = [0, 1]*2 + [1000000]*20

        def faketime():
            return rets.pop(0)

        realtime = time.time
        time.time = faketime
        realip = socket.gethostbyname
        socket.gethostbyname = fakeip
        
        testcase = "http://12.34.56.78/"
        self.scheduler.put_blacklist(testcase)
        ret = self.scheduler.get()
        self.assertEquals(ret, None)
        ret = self.scheduler.get()
        self.assertEquals(ret, testcase)

        time.time = realtime
        socket.gethostbyname = realip

    def test_put(self):
        realip = socket.gethostbyname
        socket.gethostbyname = fakeip

        testcase = "http://12.34.56.78/"
        self.scheduler.put(testcase)
        ret = self.queue.consume()
        self.assertEquals(ret, None)
        testcase = "http://143.248.204.86/"
        self.scheduler.put(testcase)
        ret = self.queue.consume()
        self.assertEquals(ret, testcase)

        socket.gethostbyname = realip

    def test_get(self):
        realip = socket.gethostbyname
        socket.gethostbyname = fakeip

        testcase = "http://12.34.56.78/"
        self.queue.append(testcase)
        ret = self.scheduler.get()
        self.assertEquals(ret, testcase)
        ret = self.scheduler.get()
        self.assertEquals(ret, None)

        socket.gethostbyname = realip

    def test_add_ip(self):
        ip = "12.34.56.78"
        time = 1234
        self.scheduler.add_ip(ip, time)
        ret = self.scheduler.get_time(ip)
        self.assertEquals(int(ret), int(time))

    def test_get_time(self):
        ret = self.scheduler.get_time("123.456.789.0")
        self.assertEquals(ret, None)

    def test_server_politeness(self):
        rets = [0, 1] + [0, 2]*3 + [0, 300000]*3

        def faketime():
            return rets.pop(0)

        A = "http://123.456.789.0/"
        B = "http://12.34.56.78/"
        self.queue.append(A)
        self.queue.append(A)
        self.queue.append(B)

        realtime = time.time
        time.time = faketime
        realip = socket.gethostbyname
        socket.gethostbyname = fakeip

        ret = self.scheduler.get()
        self.assertEquals(ret, A)
        ret = self.scheduler.get()
        self.assertEquals(ret, B)
        ret = self.scheduler.get()
        self.assertEquals(ret, A)

        time.time = realtime
        socket.gethostbyname = realip

def suite():
    return unittest.TestLoader().\
        loadTestsFromTestCase(SchedulerTestCase)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())   #pragma: no cover
