#!/usr/bin/env python
'''
DocIndexer HTML parser.
'''
import re
import logging

from BeautifulSoup import *

from ksearch import util

class HtmlParser(object):

    description = 'HTML document parser'
    mime_types = ['text/html']

    def __init__(self):
        self.logger = logging.getLogger("parser")

    def strip_comments(self, string):
        return re.sub(ur'<!--.*?-->', u'', string, re.DOTALL)

    def extract_text(self, element, content=''):
        if not element: return None

        if isinstance(element, NavigableString):
            if isinstance(element, Comment):
                return None
            value = self.strip_comments(element.string)
            value = value.strip()
            if value:
                return value + u" "
        elif element.string:
            if re.match(ur"^(p|td|div)$", element.name, re.I):
                postfix = u"\n"
            else:
                postfix = u" "

            if re.match(ur"^(script|style)$", element.name, re.I):
                return None
            
            # XXX: BeautifulSoup hack to parse table in table
            if re.match(ur"<table", element.string, re.I):
                soup = BeautifulSoup(element.string)
                return self.extract_text(soup) + "\n"

            value = element.string.strip()
            if value:
                return value + postfix
        else:
            if re.match(ur"^(p|td|div)$", element.name, re.I):
                postfix = u"\n"
            else:
                postfix = u" "

            its = (self.extract_text(x) for x in element if x)
            ret = u"".join(x for x in its if x) + postfix
            return util.decode_htmlentities(ret)

    def extra_normalize(self, text):
        if not text: return None
        ret = text
        return ret
        #ret = re.sub(ur'([\[\]\{\}\:\;\'\"\<\>\?\.\,\-\_\+\=\|\\\~\`\!\@\#\$\%\^\&\*\(\)])', ur' \1 ', text)
        #ret = re.sub(ur'   ', u' ', ret)
        #ret = re.sub(u'\n\n\n\n\n', u'\n', ret)
        ret = re.sub(u'\n *\n', u'\n', ret)
        ret = re.sub(u'\n *\n', u'\n', ret)
        ret = re.sub(u'\n\n\n', u'\n', ret)
        return ret

    def text_from_file(self,filename):
        f = open(filename)
        ret = self.text(f.read())
        f.close()
        return ret

    def text(self,src):
        soup = BeautifulSoup(src.replace("\n",""))
        try:
            title_tag = soup.html.head.title
            if isinstance(title_tag, unicode):
                self._title = title_tag
            else:
                self._title = self.extract_text(title_tag)
            self.logger.debug("Parsed title: %s", repr(self._title))
        except Exception:
            self._title = u"Untitled Document"

        if not isinstance(self._title, unicode):
            self._title = u"Untitled Document"
        else:
            self._title = self._title.strip()
        if not self._title:
            self._title = u"Untitled Document"

        return self.extra_normalize(
                self.extract_text(soup))

    def title(self):
        try:
            return self._title
        except Exception:
            return u"Title Not Parsed Yet"

# vim: ts=4 sw=4:
