#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import absolute_import

import Pyro.core
import Pyro.naming
import Pyro.util
import Pyro.errors

import optparse
import sys
import bsddb
import os

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from ksearch.crawler.queue import BTreeQueue
from ksearch.crawler.scheduler import Scheduler

if __name__ == '__main__':
    logger = Pyro.util.UserLogger()

    source_name = "CrawlerServer"

    logger.msg(source_name, "Welcome to Crawler Server, initializing...")
    parser = optparse.OptionParser()
    parser.add_option("-n", "--namespace", dest="object_namespace", default="ksearch_main",
                      help="Object name to register in NS")
    parser.add_option("", "--queue_path", dest="queue_path", default="queue_db",
                      help="The path to save the queue")
    parser.add_option("", "--index_queue_path", dest="idx_queue_path", default="idx_queue_db",
                      help="The path to save the index queue: queue to index the crawled files")
    parser.add_option("", "--scheduler_path", dest="scheduler_path", default="scheduler_db",
                      help="The scheduler path")
    options, args = parser.parse_args()
    logger.msg(source_name, "Args parsed:", repr(options.__dict__))

    # 없는 디렉토리를 만든다
    def create_dir(path):
      if not os.path.isdir(path):
        logger.msg(source_name, "Directory %s does not exist, creating.."%path)
        os.makedirs(path)

    create_dir(options.queue_path)
    create_dir(options.idx_queue_path)
    create_dir(options.scheduler_path)

    logger.msg(source_name, "Finding nameserver...")
    Pyro.core.initServer(banner=0)
    ns=Pyro.naming.NameServerLocator().getNS()
    try:
        ns.createGroup(":%s"%options.object_namespace)
    except Pyro.errors.NamingError:
        pass

    daemon=Pyro.core.Daemon()
    daemon.useNameServer(ns)

    storage = Pyro.core.getProxyForURI("PYRONAME://:%s.storage"%options.object_namespace)
    storage._setOneway(['modify'])

    database = Pyro.core.getProxyForURI("PYRONAME://:%s.database"%options.object_namespace)

    logger.msg(source_name, "Creating queue...")
    pobj = Pyro.core.ObjBase()
    queue = BTreeQueue(options.queue_path, database)
    pobj.delegateTo(queue)
    uri=daemon.connectPersistent(pobj, 
            ':%s.%s'%(options.object_namespace, 'urlqueue'))

    logger.msg(source_name, "Creating index queue...")
    pobj = Pyro.core.ObjBase()
    idx_queue = BTreeQueue(options.idx_queue_path, database)
    pobj.delegateTo(idx_queue)
    uri=daemon.connectPersistent(pobj, 
            ':%s.%s'%(options.object_namespace, 'idxqueue'))

    logger.msg(source_name, "Creating scheduler...")
    pobj = Pyro.core.ObjBase()
    pobj.delegateTo(Scheduler(queue, storage, database, options.scheduler_path))
    uri=daemon.connectPersistent(pobj, 
            ':%s.%s'%(options.object_namespace, 'scheduler'))

    def clean_up(signal, frame):
        logger.msg(source_name, "Cleaning up crawler server...")
        idx_queue.close()
        queue.close()
        daemon.shutdown(True)
        sys.exit(0)

    import signal
    signal.signal(signal.SIGTERM, clean_up)
    signal.signal(signal.SIGQUIT, clean_up)

    logger.msg(source_name, "Request Loop!")
    try:
        daemon.requestLoop()
    finally:
        logger.msg(source_name, "Shutting down daemon...")
        daemon.shutdown(True)
