#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import absolute_import

import Pyro.core
import Pyro.naming
import Pyro.util
import Pyro.errors

import optparse
import sys
import bsddb
import os

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from ksearch.crawler.database import Database

if __name__ == '__main__':
    logger = Pyro.util.UserLogger()

    source_name = "CrawlerServer"

    logger.msg(source_name, "Welcome to Database Server, initializing...")
    parser = optparse.OptionParser()
    parser.add_option("-n", "--namespace", dest="object_namespace", default="ksearch_main",
                      help="Object name to register in NS")
    parser.add_option("", "--database_path", dest="database_path", default="database_db",
                      help="The database path")
    options, args = parser.parse_args()
    logger.msg(source_name, "Args parsed:", repr(options.__dict__))

    # 없는 디렉토리를 만든다
    def create_dir(path):
      if not os.path.isdir(path):
        logger.msg(source_name, "Directory %s does not exist, creating.."%path)
        os.makedirs(path)

    create_dir(options.database_path)

    logger.msg(source_name, "Finding nameserver...")
    Pyro.core.initServer(banner=0)
    ns=Pyro.naming.NameServerLocator().getNS()
    try:
        ns.createGroup(":%s"%options.object_namespace)
    except Pyro.errors.NamingError:
        pass

    daemon=Pyro.core.Daemon()
    daemon.useNameServer(ns)

    logger.msg(source_name, "Creating database...")
    pobj = Pyro.core.ObjBase()
    database = Database(options.database_path)
    pobj.delegateTo(database)
    uri=daemon.connectPersistent(pobj, 
            ':%s.%s'%(options.object_namespace, 'database'))

    def clean_up(signal, frame):
        logger.msg(source_name, "Cleaning up database server...")
        database.close()
        daemon.shutdown(True)
        sys.exit(0)

    import signal
    signal.signal(signal.SIGTERM, clean_up)
    signal.signal(signal.SIGQUIT, clean_up)

    logger.msg(source_name, "Request Loop!")
    try:
        daemon.requestLoop()
    finally:
        logger.msg(source_name, "Shutting down daemon...")
        daemon.shutdown(True)
