#!/usr/bin/env python

from __future__ import absolute_import

import thread_hack
import Pyro.core
import Pyro.naming
import Pyro.util

import optparse
import sys
import bsddb
import os
import threading
import time

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from ksearch.crawler.queue import BTreeQueue
from ksearch.crawler.scheduler import Scheduler
from ksearch.crawler.downloader import Downloader

if __name__ == '__main__':
    Pyro.core.initClient(banner=0)

    logger = Pyro.util.UserLogger()

    source_name = "Downloader"

    logger.msg(source_name, "Welcome to Downloader, initializing...")
    parser = optparse.OptionParser()
    parser.add_option("-n", "--namespace", dest="object_namespace", default="ksearch_main",
                      help="Object name to register in NS")
    parser.add_option("-t", "--threads", dest="threads", default=10,
                      help="Number of downloader threads to run")
    options, args = parser.parse_args()
    logger.msg(source_name, "Args parsed:", repr(options.__dict__))

    logger.msg(source_name, "Finding nameserver...")
    Pyro.core.initServer()
    ns=Pyro.naming.NameServerLocator().getNS()
    daemon=Pyro.core.Daemon()
    daemon.useNameServer(ns)

    def clean_up(signal, frame):
        logger.msg(source_name, "Cleaning up crawler...")
        sys.exit(0)

    import signal
    signal.signal(signal.SIGTERM, clean_up)
    signal.signal(signal.SIGQUIT, clean_up)

    logger.msg(source_name, "Creating downloader...")
    scheduler = Pyro.core.getProxyForURI("PYRONAME://:%s.scheduler"%options.object_namespace)
    scheduler._setOneway(['put', 'put_blacklist'])
    storage = Pyro.core.getProxyForURI("PYRONAME://:%s.storage"%options.object_namespace)
    storage._setOneway(['modify'])
    idx_queue = Pyro.core.getProxyForURI("PYRONAME://:%s.idxqueue"%options.object_namespace)
    idx_queue._setOneway(['append', 'prepend'])
    database = Pyro.core.getProxyForURI("PYRONAME://:%s.database"%options.object_namespace)
    import copy

    class DownloaderThread(threading.Thread):
        def run(self):
            while True:
                try:
                    scheduler_ = copy.copy(scheduler)
                    storage_ = copy.copy(storage)
                    database_ = copy.copy(database)
                    idx_queue_ = copy.copy(idx_queue)
                    d = Downloader(scheduler_, storage_, database_, idx_queue_)
                    d.start()
                    sys.exit(0)
                except KeyboardInterrupt:
                    sys.exit(0)
                except Exception, x:
                    logger.error("Downloader Error:\n" + ''.join(Pyro.util.getPyroTraceback(x)))
                    logger.error("Downloader error. Exiting now...")
                    sys.exit(1)

    threads = []
    for i in range(options.threads):
        thread = DownloaderThread()
        thread.start()
        threads.append(thread)
        time.sleep(5)

    for thread in threads:
        thread.join()
