#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

import optparse
import sys
import os
import copy

import thread_hack
import Pyro.core
import Pyro.naming
import Pyro.util
import Pyro.errors
import Pyro

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

if __name__ == '__main__':
    Pyro.core.initClient(banner=0)

    logger = Pyro.util.UserLogger()

    index_dir = "test_index"
    source_name = "IndexerService"

    logger.msg(source_name, "Welcome to Indexer Service Server, initializing...")
    parser = optparse.OptionParser()
    parser.add_option("-n", "--namespace", dest="object_namespace", default="ksearch_main",
                      help="Object name to register in NS [default: %default]")

    backends = ('lucene', 'xapian')
    backends_help = ('Search backend to use (%s) [default: %%default]' %
        ', '.join(backends))
    parser.add_option("--backend", dest="backend",
        choices=backends, default="lucene", help=backends_help)

    options, args = parser.parse_args()
    logger.msg(source_name, "Args parsed:", repr(options.__dict__))

    logger.msg(source_name, "Finding nameserver...")
    Pyro.core.initServer(banner=0)
    ns = Pyro.naming.NameServerLocator().getNS()
    try:
        ns.createGroup(":%s"%options.object_namespace)
    except Pyro.errors.NamingError:
        pass

    daemon=Pyro.core.Daemon()
    daemon.useNameServer(ns)

    database = Pyro.core.getProxyForURI(
            "PYRONAME://:" + options.object_namespace + ".database")
    database_getter = lambda: copy.copy(database)

    logger.msg(source_name, "Creating IndexerService...")
    pobj = Pyro.core.ObjBase()

    if options.backend == 'lucene':
        from ksearch.search.indexerservice import IndexerService
    elif options.backend == 'xapian':
        from ksearch.search.xapian_backend import IndexerService

    indexer_service = IndexerService(index_dir, database_getter)
    pobj.delegateTo(indexer_service)
    uri=daemon.connectPersistent(pobj, 
            ':%s.%s'%(options.object_namespace, 'indexer_service'))

    def clean_up(signal, frame):
        logger.msg(source_name, "Cleaning up indexer service...")
        daemon.shutdown(True)
        # XXX: 가끔 호출을 안하는 경우가 있다.
        indexer_service._close()
        sys.exit(0)

    import signal
    signal.signal(signal.SIGTERM, clean_up)
    signal.signal(signal.SIGQUIT, clean_up)

    logger.msg(source_name, "Request Loop!")
    try:
        daemon.requestLoop()
    finally:
        logger.msg(source_name, "Shutting down daemon...")
        daemon.shutdown(True)
