# -*- coding: utf-8 -*-
import sys
import os
import unittest
import ham
import re

DIC_DIR = os.path.join(os.path.dirname(ham.__file__), "hdic")
DIC_FILE = os.path.join(DIC_DIR, "KLT2000.ini")

class HamTestCase(unittest.TestCase):

    def setUp(self):
        ## XXX: Decent hack to bypass the dict directory path settings
        f = open(DIC_FILE+".template", "r")
        cont = f.read()
        f.close()
        cont = re.sub(r"DicDirPath=[\-\.\/a-zA-Z]+", 
                      "DicDirPath="+DIC_DIR, cont)
        f = open(DIC_FILE, "w")
        f.write(cont)
        f.close()

        self.ham = ham.ham_open_index("V", DIC_FILE)

    def tearDown(self):
        ham.ham_close_index(self.ham)

    def test_ham_get_stems(self):
        str = u'나는 하늘을 나는 사람'
        result = [(0, '1', u'\ub098'), (0, 'V', u'\ub098v'), 
                  (0, 'V', u'\ub0a0v'), (1, 'N', u'\ud558\ub298'), 
                  (2, '1', u'\ub098'), (2, 'V', u'\ub098v'), 
                  (2, 'V', u'\ub0a0v'), (3, 'N', u'\uc0ac\ub78c')]
        ss = ham.ham_get_stems(str, self.ham)
        self.assertEqual(ss, result)

    def test_ham_get_stems_merged(self):
        str = u'나는 하늘을 나는 사람'
        result = [[('1', u'\ub098'), ('V', u'\ub098v'), 
                   ('V', u'\ub0a0v')], [('N', u'\ud558\ub298')], 
                  [('1', u'\ub098'), ('V', u'\ub098v'), 
                   ('V', u'\ub0a0v')], [('N', u'\uc0ac\ub78c')]]
        ss = ham.ham_get_stems_merged(str, self.ham)
        self.assertEqual(ss, result)

def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(HamTestCase)
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
