# -*- coding: utf-8 -*-

import os
import logging
import sys

from koreanstemfilter import KoreanStemFilter
from sentencetokenizer import SentenceTokenizer
import lucene

class KoreanAnalyzer(lucene.PythonAnalyzer):
    """KoreanAnalyzer, can be used with english too.
    Example:
    >>> import lucene
    >>> ka = KoreanAnalyzer()
    >>> # 나는 하늘을 나는 super 사람 unicode encoded
    >>> str = u"\\ub098\\ub294 \\ud558\\ub298\\uc744 \\ub098\\ub294 super \\uc0ac\\ub78c"
    >>> ts = ka.tokenStream('content', lucene.StringReader(str))
    >>> ts.next().toString()
    u'(\\ub098,0,2)'
    >>> ts.next().termText()
    u'\\ub098v'
    >>> while True:
    ...     tok = ts.next()
    ...     if not tok: break
    ...     print tok.toString().encode('utf-8')
    (날v,0,2,posIncr=0)
    (하늘,3,6)
    (나,7,9)
    (나v,7,9,posIncr=0)
    (날v,7,9,posIncr=0)
    (super,10,15)
    (사람,16,18)
    >>> str[7:9]
    u'\\ub098\\ub294'
    >>> str[10:15]
    u'super'
    """
    def __init__(self):
        lucene.PythonAnalyzer.__init__(self)
        self.logger = logging.getLogger("ksearch.search.analysis")
        self.logger.info("Initializing ham...")


    def tokenStream(self, fieldName, reader):
        #result = StandardTokenizer(reader)
        #result = StandardFilter(result)
        #result = LowerCaseFilter(result)
        #result = PorterStemFilter(result)
        ##result = SentenceTokenizer(reader)
        result = SentenceTokenizer(reader)
        result = KoreanStemFilter(result)
        #result = StopFilter(result, StopAnalyzer.ENGLISH_STOP_WORDS)

        return result

if __name__ == '__main__':
    import doctest
    doctest.testmod()
