#-*- coding: utf-8 -*-
import re
import ham
import os
import logging
import threading

import lucene
DIC_DIR = os.path.join(os.path.dirname(ham.__file__), "hdic")
DIC_FILE = os.path.join(DIC_DIR, "KLT2000.ini")

HAM = None

def get_ham():
    global HAM
    if HAM is None:
        # Initializing HAM
        ## XXX: Decent hack to bypass the dict directory path settings
        f = open(DIC_FILE+".template", "r")
        cont = f.read()
        f.close()
        cont = re.sub(r"DicDirPath=[ \-\.\/a-zA-Z]+", 
                      "DicDirPath="+DIC_DIR, cont)
        f = open(DIC_FILE, "w")
        f.write(cont)
        f.close()

        HAM = ham.ham_open_index("V", DIC_FILE)
        if not HAM:
            logging.critical("Ham init failed")
            raise Exception("Ham init failed!")
    return HAM

get_ham()

class KoreanStemFilter(lucene.PythonTokenFilter):

    def __init__(self, tokenStream):
        lucene.PythonTokenFilter.__init__(self, tokenStream)
        self.logger = logging.getLogger("search")

        # Initializing members
        self.input = tokenStream
        self.token_list = []
        self.sentence_tokens = []
        self.korean_regex = re.compile(u"[가-힣ㄱ-ㅎㅏ-ㅣ]")
        self.eng_normalize_regex = re.compile(u"[^a-zA-Z0-9]")

        self.start_offset = -1
        self.end_offset = -1

        self.ham = get_ham()

    def next(self):
        if not self.token_list:
            # Convert input into sentence_tokens
            for token in self.input:
                token_text = token.termText()
                token_start = token.startOffset()
                token_end = token.endOffset()
                token_posincr = token.getPositionIncrement()

                if self.korean_regex.search(token_text):
                    # If Korean, use HAM
                    # To avoid HAM's segfault, crop sentence
                    if len(token_text) > 35:
                        self.logger.warning("Token cropping - %s_crop_%s", token_text[:35], token_text[35:])
                        token_text = token_text[:35]

                    ham_lock = threading.Lock()
                    ham_lock.acquire()
                    analyzed_stems = ham.ham_get_stems(token_text, self.ham)
                    ham_lock.release()

                    if not analyzed_stems:
                        # XXX : Implement handling POS_INCREMENT
                        continue
                    #print repr(analyzed_stems)
                    if analyzed_stems:
                        token = lucene.Token(analyzed_stems[0][2], token_start, token_end)
                        token.setPositionIncrement(token_posincr)
                        self.token_list.append(token)

                        if len(analyzed_stems) > 1:
                            for stem in analyzed_stems[1:]:
                                token = lucene.Token(stem[2], token_start, token_end)
                                token.setPositionIncrement(0)
                                self.token_list.append(token)
                else:
                    # If Non-Korean, use Pre-made filters
                    class DummyTokenStream(lucene.PythonTokenStream):
                        def __init__(self, token_list):
                            super(DummyTokenStream, self).__init__(self)
                            self.token_list_iter = iter(token_list)

                        def next(self):
                            return self.token_list_iter.next()

                    result = lucene.StandardFilter(DummyTokenStream([token]))
                    result = lucene.LowerCaseFilter(result)
                    result = lucene.PorterStemFilter(result)
                    result = lucene.StopFilter(result,
                                        lucene.StopAnalyzer.ENGLISH_STOP_WORDS)
                    for x in result:
                        self.token_list.append(x)

        if not self.token_list:
            return None
        ret = self.token_list.pop(0)
        return ret
