# -*- coding: utf-8 -*-

import os
import codecs
import logging
import time
import traceback
import re

from ksearch.parser.htmlparser import HtmlParser
from ksearch.parser.pdfparser import PdfParser

import Pyro.errors

class DocumentIndexer(object):
    def __init__(self, indexer_service, storage, index_queue):
        self.indexer_service = indexer_service
        self.index_queue = index_queue
        self.storage = storage
        self.html_parser = HtmlParser()
        self.pdf_parser = PdfParser()
        self.logger = logging.getLogger("search")

    def start(self):
        while True:
            doc_id = self.index_queue.consume()
            if doc_id == None:
                time.sleep(1)
                continue
            doc_id = int(doc_id)

            raw_data = self.storage.get_data(doc_id)
            type = self.storage.get_type(doc_id)
            url = self.storage.get_url(doc_id)

	    pagerank = self.storage.get_pagerank(doc_id)
	    if not pagerank: pagerank = 1
	    pagerank = float(pagerank)

            try:
                if type == 'text/html':
                    text = self.html_parser.text(raw_data)
                    title = self.html_parser.title()
                elif type == 'application/pdf':
                    text = self.pdf_parser.text(raw_data)
                    title = url
                    # XXX: Not yet supported
                    # title = self.pdf_parser.title()
                else:
                    continue
            except Exception, e:
                self.logger.error("Error parsing document %s\nDetails below:\n%s" % \
                                  (url, traceback.format_exc()))
                continue
            if not text:
                continue
            if text.strip() == '':
                continue

            if title == u"Untitled Document":
                self.logger.info("Untitled Document: %s", url)

            # Remove whitespace
            text = text.strip()
            text = ' '.join(x for x in text.split() if x)

            # Crop if large
            MAX_TEXT_SIZE = 1024*1
            if len(text) > MAX_TEXT_SIZE:
                self.logger.info("Document %s large. Cropping", url)
                text = text[:MAX_TEXT_SIZE]
            file_type = type.rsplit('/', 1)[-1]
            #type : MIME type  // file_type : file extension

            try:
                self.indexer_service.index("web", url, title, text, boost=(1.0 * pagerank), filetype=file_type)
            except Pyro.errors.ConnectionClosedError,x:  # or possibly even ProtocolError
                self.logger.error("Connection lost. REBINDING...")
                self.logger.warn("Restart the server now")
                self.indexer_service.rebindURI(tries=10000, wait=10)
                self.logger.info("Connection has been recovered")
            except Exception, x:
                self.logger.critical(''.join(Pyro.util.getPyroTraceback(x)))
