# -*- coding: utf-8 -*-

import unittest
import os
import codecs
import logging
import tempfile

from time import time
from datetime import timedelta
from lucene import *

from ksearch.search.analysis.koreananalyzer import KoreanAnalyzer
from ksearch.search.analysis.koreanstemfilter import KoreanStemFilter

class IndexerSearcherTestCase(unittest.TestCase):

    def test_indexersearcher(self):

        #indexDir = 'testindex'
        indexDir = tempfile.mkdtemp()
        dataDir = os.path.join(os.path.dirname(__file__), 'text')

        start = time()
        numIndexed = self.index(indexDir, dataDir)
        duration = timedelta(seconds=time() - start)

        logging.info("Indexing %s files took %s", numIndexed, duration)
        self._testsearch(indexDir)
        if not (os.path.exists(indexDir) and os.path.isdir(indexDir)):
            raise IOError, "%s does not exist or is not a directory" \
                    %(indexDir)

        import shutil
        shutil.rmtree(indexDir)

    def _testsearch(self, indexDir):
        test_cases = [(u'임금', 1, 'honggildong'), 
                      (u'test', 0, ''),
                      (u'임금', 1, 'honggildong'),
                      (u'english', 1, 'wikipedia'),
                      (u'english', 1, 'wikipedia'),
                      ]
        fsDir = FSDirectory.getDirectory(indexDir, False)
        searcher = IndexSearcher(fsDir)
        kanal = KoreanAnalyzer()
        queryparser = QueryParser("contents", kanal)
        for tc in test_cases:
          self.__testsearch(searcher, queryparser, tc[0], tc[1], tc[2])
        #del kanal

    def __testsearch(self, searcher, queryparser, q, num, filename):
        start = time()
        query = queryparser.parse(q)
        hits = searcher.search(query)
        duration = timedelta(seconds=time() - start)

        self.assertEquals(len(hits), num)

        if len(hits) > 0:
            hit_filename = hits[0].get("path")
            self.assertNotEquals(hit_filename.find(filename), -1)

    def index(self, indexDir, dataDir):

        if not (os.path.exists(dataDir) and os.path.isdir(dataDir)):
            raise IOError, "%s does not exist or is not a directory" \
                    %(dataDir)

        kanal = KoreanAnalyzer()

        writer = IndexWriter(indexDir, kanal, True)
        writer.setUseCompoundFile(False)

        self.indexDirectory(writer, dataDir)

        numIndexed = writer.docCount()
        self.assertEquals(numIndexed, 3)
        writer.optimize()
        writer.close()

        #del kanal

        return numIndexed

    def indexDirectory(self, writer, dir):

        for name in os.listdir(dir):
            path = os.path.join(dir, name)
            if os.path.isfile(path):
                if path.endswith('.txt'):
                    self.indexFile(writer, path)
            elif os.path.isdir(path):
                self.indexDirectory(writer, path)

    def indexFile(self, writer, path):

        try:
            file = open(path)
        except IOError, e:
            logging.error('IOError while opening %s: %s', path, e)
        else:
            logging.info('Indexing %s', path)
            doc = Document()
            f = codecs.open(path, 'r', 'utf-8', 'replace')
            doc.add(Field("contents", f.read(),
                          Field.Store.YES, Field.Index.TOKENIZED))
            doc.add(Field("path", os.path.abspath(path),
                          Field.Store.YES, Field.Index.UN_TOKENIZED))
            writer.addDocument(doc)
            file.close()
            f.close()

def suite():
    return unittest.TestLoader().\
        loadTestsFromTestCase(IndexerSearcherTestCase)

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())   #pragma: no cover
