#-*- coding: utf-8 -*-
import re
import urlparse
import urllib
import Pyro.core
from email.Utils import formatdate

def smart_unicode(txt, candidates=['cp949', 'utf-8', 'iso-8859-1']):
    r"""Smartly converts various encodings...
    Example:
    >>> smart_unicode("exception\xa0error")
    u'exception\uc7e2rror'
    >>> # should be
    >>> #u'exception\xa0error'
    >>> smart_unicode('\xec\x95\x88\xeb\x85\x95\xed\x95\x98\xec\x84\xb8\xec\x9a\x94.')
    u'\uc548\ub155\ud558\uc138\uc694.'
    >>> smart_unicode('\x97\xcd\x8dZ\x84\xb8\x85\xf1.')
    u'\uc00d\ub6dd\uaf6d\ub08b.'

    """
    if isinstance(txt, unicode): return txt
    if not isinstance(txt, str): txt = str(str)

    #if '\xA0' in list(txt):
    #    candidates = ['iso-8859-1', 'cp949', 'utf-8']

    for enc in candidates:
        try:
            txt = txt.decode(enc)
        except Exception:
            pass
        else:
            return txt

    for enc in candidates:
        try:
            txt = txt.decode(enc, 'replace')
        except Exception:
            pass
        else:
            return txt


def extract_links(data, base_url=""):
    next = []
    next.extend(re.findall(r'(?:src|href|url)="([^"]+)"', data))
    next.extend(re.findall(r"(?:src|href|url)='([^']+)'", data))
    next.extend(re.findall(r"""(?:src|href|url)=([^ "'/>]+)""", data))
    next.extend(re.findall("(https?:[A-Za-z0-9/](([A-Za-z0-9$_.+!*,;/?:@&~=-])|%[A-Fa-f0-9]{2}){1,333}(#([a-zA-Z0-9][a-zA-Z0-9$_.+!*,;/?:@&~=%-]{0,1000}))?)", data))

    result = set()

    for item in next:
        if isinstance(item, tuple): item = item[0]
        item = item.strip()
        if base_url:
            next_url = urlparse.urljoin(base_url, item)

        # unicodify
        next_url = smart_unicode(next_url)
        # &amp; -> & and etc..
        next_url = decode_htmlentities(next_url)
        # Remove http://asdf/asdf"#aaa" <-- this
        next_url = next_url.rsplit("#", 1)[0]

        result.add(next_url.encode('utf-8'))
    return result 


def decode_htmlentities(string):
    def substitute_entity(match):
        from htmlentitydefs import name2codepoint as n2cp
        try:
            ent = match.group(2)
            if match.group(1) == "#":
                if ent.startswith("x"):
                    return unichr(int(ent[1:], 16))
                else:
                    return unichr(int(ent))
            else:
                cp = n2cp.get(ent)

                if cp:
                    return unichr(cp)
                else:
                    return match.group()
        except ValueError, e:
            return match.group()

    entity_re = re.compile("&(#?)(\d{1,5}|\w{1,8});")
    return entity_re.subn(substitute_entity, string)[0]

def get_object(obj_name, namespace='test_world'):
    uri = 'PYRONAME://:' + namespace + '.' + obj_name
    return Pyro.core.getProxyForURI(uri)

def query2dict(string):    
    """HTTP Query 정보를 dictionary 형태로 가공한다.
    예를 들면 string='q=##&p=**' -> d['q']=## d['p']=** 과 같이 바뀐다.
    """
    
    d = dict()
    items = string.split("&")
    for i in items:
        item_list = i.split("=")
        try:
            k = item_list[0]
            v = item_list[1].replace('+', ' ')
            v = urllib.unquote(v).decode('utf8')
            d[k] = v
        except IndexError:
            d['q'] = ''  #d[]// 'q' : query // 'p' : page number
    return d

def raw(string):
    """string 을 태그 사이에서 출력 가능한 형태로 변환한다.
        ex) <hr> -> &lt;hr&gt;

       참고로, Django 템플릿 엔진을 이용하게 되면 기본적으로
       자동 escape를 해주기 때문에 이 함수를 사용할 일이 없어질
       것이다.

       생짜 HTML로 바로 출력하고 싶다면 템플릿에서 safe 필터를
       사용하면 된다.
    """

    raw_str = ""
    for i in string:
        if i == '<':
            raw_str += '&lt;'
        elif i == '>':
            raw_str += '&gt;'
        else:
            raw_str += i
    return raw_str

def http_date(epoch_seconds):
    """RFC1123 형식에 맞춘 날짜 문자열로 변환한다."""

    rfcdate = formatdate(epoch_seconds)
    return '%s GMT' % rfcdate[:25]


