import socket
import xmlrpclib
import urlparse

HOST = 'localhost'

def is_ubigraph_available():
    s = socket.socket()
    try:
        s.connect((HOST, 20738))
        return True
    except socket.error:
        return False

class GraphAPI:
    def __init__(self): pass
    def new_url(self, url): pass
    def new_link(self, url_from, url_to): pass
    def label_url(self, url): pass
    def size_url(self, url, size): pass

class UbiGraph:

    def __init__(self):
        self.url_id = {}
        self.link_id = {}

        server = xmlrpclib.Server('http://'+HOST+':20738/RPC2')
        G = server.ubigraph
        G.clear()
        G.set_vertex_style_attribute(0, 'color', '#ffff00')
        G.set_vertex_style_attribute(0, 'shape', 'octahedron')
        G.set_vertex_style_attribute(0, 'size', '0.8')
        G.set_vertex_style_attribute(0, 'fontsize', '5')
        G.set_edge_style_attribute(0, 'color', '#ffffff')
        G.set_edge_style_attribute(0, 'showstrain', 'true')
        G.set_edge_style_attribute(0, 'strength', '5')
        self.G = G

    def new_url(self, url):
        if url in self.url_id:
            return
        id = self.G.new_vertex()
        self.url_id[url] = id

    def new_link(self, url_from, url_to):
        if (url_from, url_to) in self.link_id:
            return
        if url_from == url_to:
            return
        if url_from not in self.url_id:
            return
        if url_to not in self.url_id:
            return

        from_id = self.url_id[url_from]
        to_id = self.url_id[url_to]
        id = self.G.new_edge(from_id, to_id)
        _, host_to, _, _, _, _ = urlparse.urlparse(url_to)
        _, host_from, _, _, _, _ = urlparse.urlparse(url_from)
        if host_to != host_from:
            self.G.set_edge_attribute(id, 'strength', '0')

        self.link_id[url_from, url_to] = id

    def label_url(self, url):
        if url not in self.url_id:
            return
        id = self.url_id[url]
        self.G.set_vertex_attribute(id, 'label', url)

    def size_url(self, url, size):
        if url not in self.url_id:
            return
        id = self.url_id[url]
        import math
        if size > 1:
            size = 1 + math.log(size, 2)
        self.G.set_vertex_attribute(id, 'size', str(size))

_graph = None
def get_graph():
    global _graph
    if _graph is not None:
        return _graph
    if is_ubigraph_available():
        _graph = UbiGraph()
        return _graph
    _graph = GraphAPI()
    return _graph
